import { Kind, } from 'graphql';
export function visitResolutionPath(document, visitFn) {
    const operations = [];
    const fragments = {};
    for (const definition of document.definitions) {
        if (definition.kind === 'OperationDefinition') {
            operations.push(definition);
        }
        else if (definition.kind === 'FragmentDefinition') {
            fragments[definition.name.value] = definition;
        }
    }
    function visitSelectionNode(selectionNode, parentPath) {
        switch (selectionNode.kind) {
            case Kind.FIELD: {
                const currentPath = [...parentPath, selectionNode.alias?.value || selectionNode.name.value];
                visitFn({
                    node: selectionNode,
                    path: currentPath,
                });
                selectionNode.selectionSet?.selections?.forEach(selection => {
                    visitSelectionNode(selection, currentPath);
                });
                break;
            }
            case Kind.INLINE_FRAGMENT: {
                selectionNode.selectionSet.selections.forEach(selection => {
                    visitSelectionNode(selection, parentPath);
                });
                break;
            }
            case Kind.FRAGMENT_SPREAD: {
                const fragmentName = selectionNode.name.value;
                if (fragmentName === '__export') {
                    break;
                }
                const fragment = fragments[fragmentName];
                if (!fragment) {
                    throw new Error(`No fragment found with name ${fragmentName}`);
                }
                fragment.selectionSet.selections.forEach(selection => {
                    visitSelectionNode(selection, parentPath);
                });
                break;
            }
        }
    }
    for (const operationNode of operations) {
        operationNode.selectionSet.selections.forEach(selection => {
            visitSelectionNode(selection, []);
        });
    }
}
